<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	//include all of the standard forms
	require_once($path."/OpenSiteAdmin/scripts/classes/Forms/Form_List.php");
	require_once($path."/OpenSiteAdmin/scripts/classes/Forms/Form_Single.php");
	
	/**
	 * Provides a template for creating HTML forms.
	 * 
	 * Handles display and processing of a form field. Also provides static
	 * data members and methods for form-related processing throughout the framework.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	abstract class Form {
		/**
		 * @static
		 * @final
		 * @var Mode constant to denote an add type form.
		 */
		const ADD = 1;
		/**
		 * @static
		 * @final
		 * @var Mode constant to denote an edit type form.
		 */
		const EDIT = 2;
		/**
		 * @static
		 * @final
		 * @var Mode constant to denote a delete type form.
		 */
		const DELETE = 3;
		
		/**
		 * @static
		 * @final
		 * @var Constant for the delimiter between the row prefix ID, fieldname,
		 *		and any field suffixes for a given field's form field name.
		 */
		const DELIMITER = '|';
		
		/** @var An array of all the rows associated with this form. */
		protected $rows;
		/** @var Options (if any) for a form. */
		protected $options;
		
		/**
		 * Constructs a new form with no rows.
		 * 
		 * @param MIXED $options Options (if any) for this form. Defaults to null.
		 */
		function __construct($options=null) {
			$this->options = $options;
			$this->rows = array();
		}
		
		/**
		 * Adds a row to this form.
		 * 
		 * @param OBJECT $row The object of type RowManager to add.
		 * @return VOID
		 */
		function addRow($row) {
			if(!($row instanceof RowManager)) {
				$error = "ERROR: Attempted to add a row $row to a form that does not inherit RowManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			$this->rows[] = $row;
		}
		
		/**
		 * Displays a form's row data in an HTML form.
		 * 
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @param ARRAY $data Array of form data in the following format:
		 *				$data[rowID][fieldName] = value
		 * @return VOID
		 */
		abstract function displayForm($mode, $data);
		
		/**
		 * Processes a form with the given data.
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @param ARRAY $data Array of form data in the following format:
		 *				$data[rowID][fieldName] = value
		 * @return BOOLEAN False if this form had errors.
		 */
		function processForm($SQL, $mode, $data) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to process a form with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			switch($mode) {
				case Form::ADD:
				case Form::EDIT:
					$type = "update";
					break;
				case Form::DELETE:
					$type = "delete";
					break;
				default:
					$msg = "FATAL ERROR: Unsupported form type ".$mode;
					print $msg;
					ErrorLogManager::log($msg, ErrorLogManager::FATAL, $SQL);
					die("This error has been logged. Please contact your system administrator.");
					break;
			}
			
			$success = true;
			foreach($this->rows as $row) {
				$success = $row->$type($SQL, $data[$row->getRowID()], $mode) && $success;
			}
			return $success;
		}
		
		/**
		 * Returns the data currently in the database for all the rows this form manages.
		 * 
		 * @param ARRAY $filters Array of Filter objects.
		 * @return ARRAY Array of data from the database for each row in the following format:
		 *				 $ret[RowID.Form::DELIMITER.fieldName] = value;
		 */
		function init($filters=array()) {
			$SQL = new DatabaseManager();
			$ret = array();
			
			foreach($this->rows as $row) {
				$ret += $row->init($SQL, $filters);
			}
			
			return $ret;
		}
		
		/**
		 * Returns the textual representation for a given mode constant.
		 * 
		 * @param INTEGER $mode One of the mode constants defined in this class.
		 * @return STRING Text corresponding to the given mode.
		 */
		static function getModeText($mode) {
			switch($mode) {
				case Form::ADD:
					return "add";
				case Form::EDIT:
					return "edit";
				case Form::DELETE:
					return "delete";
				default:
					print "ERROR: Form type ".$mode." is not supported<br>";
					return "";
			}
		}
		
		/**
		 * Returns any options associated with this form.
		 * 
		 * @return MIXED Form options.
		 */
		function getOptions() {
			return $this->options;
		}
	}
?>