<?php
	/*
	 *	Copyright 2007 John Oren
	 *	
	 *	Licensed under the Apache License, Version 2.0 (the "License");
	 *	you may not use this file except in compliance with the License.
	 *	You may obtain a copy of the License at
	 *		http://www.apache.org/licenses/LICENSE-2.0
	 *	Unless required by applicable law or agreed to in writing, software
	 *	distributed under the License is distributed on an "AS IS" BASIS,
	 *	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	 *	See the License for the specific language governing permissions and
	 *	limitations under the License.
	 */
	
	
	/**
	 * Handles field processing and database interaction with a row in a MySQL database table.
	 * 
	 * @author John Oren
	 * @version 1.0 December 30, 2007
	 */
	class RowManager {
		/** @var The next unique identifier for a new row (no relation to any database value). */
		private static $ID = 1;
		
		/** @var The unique identifier for this row (no relation to any database value). */
		private $rowID;
		/** @var The name of the database table this row is in. */
		private $tableName;
		/** @var The primary key field for the table this row is in. */
		private $primaryKeyField;
		/** @var Value of the primary key for this row. */
		private $primaryKey;
		/** @var FieldManager object that handles display and processing for this row's primary key. */
		private $primaryKeyObject;
		
		/** @var Array of FieldManager objects for this row. */
		private $fields;
		/** @var False if none of the fields in this row encountered errors. */
		private $errors;
		/** @var True if every field (except foreign key fields) in this row is empty. */
		private $isEmpty;
		
		/**
		 * Constructs a new row manager with no fields, a unique ID, and the given data.
		 * 
		 * @param STRING $tableName Name of the table this row corresponds to.
		 * @param STRING $primaryKeyField Name of the primary key field for the provided table.
		 */
		function __construct($tableName, $primaryKeyField) {
			$this->tableName = $tableName;
			$this->primaryKeyField = $primaryKeyField;
			$this->primaryKey = -1;
			$this->rowID = RowManager::$ID++;
			$this->fields = array();
		}
		
		/**
		 * Adds a field to the row with this row's unique ID as a prefix to the field name.
		 * 
		 * @param OBJECT $field FieldManager oject to add to this row.
		 * @return VOID
		 */
		function addField($field) {
			if($field instanceof FieldManager) {
				$field->setNamePrefix($this->getRowID());
				$this->fields[] = $field;
			} else {
				if(!is_object($field)) {
					$error = "Attempted to add a field $field that is not an object to ".$this->getTableName();
				} else {
					$error = "Attempted to add a field ".get_class($field)." that does not inherit FieldManager to ".$this->getTableName();
				}
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
		}
		
		/**
		 * Adds the given array of fields to the row by adding each field individually.
		 * 
		 * @param ARRAY $fields Array of FieldManager objects.
		 * @return VOID
		 */
		function addAllFields($fields) {
			foreach($fields as $field) {
				$this->addField($field);
			}
		}
		
		/**
		 * Returns the name of the database table this row interacts with.
		 * 
		 * @return STRING Name of this row's database table.
		 */
		function getTableName() {
			return $this->tableName;
		}
		
		/**
		 * Returns the name of the primary key field for the table this row interacts with.
		 * 
		 * @return STRING Primary key field name.
		 */
		function getPrimaryKeyField() {
			return $this->primaryKeyField;
		}
		
		/**
		 * Returns an array of all the fields this database interacts with
		 * (note that the objects in this array are passed by reference).
		 * 
		 * @return STRING Array of FieldManager objects for this row.
		 */
		function getFields() {
			return $this->fields;
		}
		
		/**
		 * Prepares to update the database with the provided set of values.
		 * If a primary key is available, the row is edited. Otherwise, the row is added.
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param ARRAY $values Array of values from _POST and _FILES in the format $values[fieldName] = value
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return BOOLEAN False if errors where encountered in any field or in processing the form.
		 */
		function update($SQL, $values, $mode) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to update a row manager with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$data = $this->getFieldData($values, $mode);
			if(isset($values["update"])) {
				return false;
			}
			if($this->isEmpty()) {
				return true;
			}
			
			if(empty($this->primaryKey)) { //add
				$isAdd = true;
				$names = "";
				$values = "";
				foreach($data as $name=>$value) {
					$names .= "`".$name."`,";
					$values .= "'".$value."',";
				}
				$names = substr($names, 0, strlen($names)-1);
				$values = substr($values, 0, strlen($values)-1);
				$query = "insert into `".$this->getTableName()."` (".$names.") values (".$values.")";
			} else { //edit
				$isAdd = false;
				$set = "";
				foreach($data as $name=>$value) {
					$set .= "`".$name."` = '".$value."',";
				}
				$set = substr($set, 0, strlen($set)-1);
				$query = "update `".$this->getTableName()."` set ".$set." where `".$this->getPrimaryKeyField()."` = '".$this->primaryKey."'";
			}
			
			return $this->checkError($query, $SQL, $isAdd);
		}
		
		/**
		 * Prepares to delete this row from the database.
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param ARRAY $values Array of values from _POST and _FILES in the format $values[fieldName] = value
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return BOOLEAN False if errors where encountered in any field or in processing the form.
		 */
		function delete($SQL, $values, $mode) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to delete a row with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			//this will set the primary key value
			$this->getFieldData($values, $mode);
			if($this->isEmpty()) {
				return true;
			}
			
			$query = "delete from `".$this->getTableName()."` where `".$this->getPrimaryKeyField()."` = '".$this->primaryKey."'";
			return $this->checkError($query, $SQL, false);
		}
		
		/**
		 * Checks for errors and runs the provided MySQL query.
		 * 
		 * @param STRING $query The MySQL query to run.
		 * @param OBJECT $SQL DatabaseManager object.
		 * @return BOOLEAN False if the row's error flag was set or if the query failed.
		 */
		protected function checkError($query, $SQL, $isAdd) {
			if($this->errors) {
				return false;
			} else {
				if(!$this->isEmpty()) {
					$result = mysql_query($query, $SQL->getLinkResource());
				}
				$error = mysql_error();
				if(!empty($error)) {
					print "There was an error processing this form.";
					ErrorLogManager::log("MYSQL ERROR -> ".$error." Table = ".$this->getTableName(), ErrorLogManager::FATAL, $SQL);
					print "This error has been logged. Please contact your system administrator.";
					return false;
				}
				if(!$this->isEmpty() && $isAdd) {
					$this->primaryKeyObject->process(mysql_insert_id($SQL->getLinkResource()), Form::ADD);
				}
				
				return true;
			}
		}
		
		/**
		 * Processes each field in this row, sets empty field errors, and checks for errors in any of the fields.
		 * 
		 * @param ARRAY $values Array of values in the format $values[fieldName] = value
		 * @param INTEGER $mode The type of this form (one of the form type constants defined in the Form class)
		 * @return ARRAY Array of processed data in the following format: $ret[fieldName] = value
		 */
		protected function getFieldData($values, $mode) {
			$ret = array();
			
			$this->errors = false;
			$this->isEmpty = true;
			foreach($this->fields as $field) {
				$fieldName = $field->getFieldName();
				$data = $values[$fieldName];
				
				$isEmpty = false;
				if(is_array($data)) {
					foreach($data as $val) {
						$isEmpty = $isEmpty || empty($val);
					}
				} else {
					$data = array();
				}
				
				if(count($data) == 1) {
					$data = $data[0];
				}
				if(count($data) == 0) {
					$data = null;
					$isEmpty = true;
				}
				
				$ret[$fieldName] = $field->process($data, $mode);
				if($fieldName == $this->primaryKeyField) {
					$this->primaryKey = $ret[$fieldName];
					$this->primaryKeyObject = $field;
				}
				
				if($isEmpty && empty($ret[$fieldName])) {
					$field->setEmpty();
				} else {
					if(get_class($field) != "ForeignKey") {
						$this->isEmpty = false;
					}
				}
				
				if($field->hasError()) {
					$this->errors = true;
				}
			}
			
			return $ret;
		}
		
		/**
		 * Returns an array of values currently in the database for this row (if any).
		 * 
		 * @param OBJECT $SQL DatabaseManager object.
		 * @param ARRAY $filters Filters to use in constructing the where clause.
		 * @param INTEGER $offset The number of the row to get from the database using ORDER BY PRIMARY_KEY_FIELD.
		 * @return ARRAY Database data in the format $row[ROWID.Form::DELIMITER.fieldName] = value
		 */
		function init($SQL, $filters, $offset=1) {
			if(!($SQL instanceof DatabaseManager)) {
				$error = "ERROR: Attempted to initialize a row manager with a database connection $SQL that does not inherit DatabaseManager";
				ErrorLogManager::log($error, ErrorLogManager::FATAL);
				die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
			}
			
			$where = RowManager::getFilterClause($filters);
			$sql = "select * from `".$this->getTableName()."` ".$where;
			$sql .= " order by `".$this->getPrimaryKeyField()."` limit ".($this->getRowID()-($offset)).",1";
			
			$result = mysql_query($sql, $SQL->getLinkResource());
			$row = mysql_fetch_assoc($result);
			
			if(!empty($row)) {
				foreach($this->getFields() as $field) {
					$fieldName = $field->getFieldName();
					$field->setDefaultValue($row[$fieldName]);
					$row[$this->getRowID().Form::DELIMITER.$fieldName] = $row[$fieldName];
					unset($row[$fieldName]);
				}
			} else {
				$row = array();
			}
			
			return $row;
		}
		
		/**
		 * Creates the MySQL where clause from all the given filters.
		 * 
		 * @param ARRAY $filters Filters to use in constructing the where clause.
		 * @return STRING MySQL where clause or an empty string if no filters were specified.
		 */
		static function getFilterClause($filters) {
			if(count($filters) == 0)
				return "";
			$ret = " WHERE ";
			
			foreach($filters as $filter) {
				if(!($filter instanceof Filter)) {
					$error = "ERROR: Attempted to generate a filter clause using a filter $filter that does not inherit Filter.";
					$error .= " Method getFilterClause static and defined in RowManager.php";
					ErrorLogManager::log($error, ErrorLogManager::FATAL);
					die("A fatal error occured - Contact your System Administrator<br>This error has been logged");
				}
				$ret .= $filter->getWhereClause()." AND ";
			}
			$ret = substr($ret, 0, -5); //remove the trailing AND
			
			return $ret;
		}
		
		/**
		 * Returns the unique identifier for this row (this value has no relation to any database value).
		 * 
		 * @return INTEGER Unique identifier for this row.
		 */
		function getRowID() {
			return $this->rowID;
		}
		
		/**
		 * Returns true if all of this row's fields where empty. Does not check foreign key fields.
		 * 
		 * @return BOOLEAN False if non foreign key data was available.
		 */
		function isEmpty() {
			return $this->isEmpty;
		}
		
		/**
		 * Called on the new row after a row has been copied using PHP's 'clone' keyword.
		 * Updates the new row's uniue identifier and performs a deep copy of this row's fields,
		 * updating field names with the new row prefix.
		 * 
		 * @return VOID
		 */
		function __clone() {
			$this->rowID = RowManager::$ID++;
			foreach($this->getFields() as $key=>$field) {
				$this->fields[$key] = clone $field;
				$this->fields[$key]->setNamePrefix($this->getRowID());
			}
		}
	}
?>